c
c-----------------------------------------------------------------------
c
      subroutine dotessel (uin, uout)
c
c.....dotessel - determination of a polyhedron that tesselates the
c     sphere.
c
      include           'implicit.inc'
      include           'error.inc'
      include           'tokens.inc'
c
      integer           uin, uout
c
c.....Polyhedra data structure:
c
      include           'polyhedron.inc'
c
      logical           ok, done
      character*(600)   line
      character*(80)    word, title, datafile
      logical           dopov, dooff, dovrml
      character*(80)    filepov, fileoff, filevrml
      integer           msteps, lp, lw, divlevel, npts, ndeg
      real*8            phaseth, phaseph, facini, isodensity
      real*8            hoffset, voffset
c
      integer           leng
      logical           finputline, setint, setdble
      character*(80)    uppcase, getword
c
c.....Define default values:
c
      msteps = 10
      facini = 0.1d0
      color_mode = color_none
      dopov = .false.
      filepov = 'tessel.pov'
      dooff = .false.
      fileoff = 'tessel.off'
      dovrml = .false.
      filevrml = 'tessel.wrl'
      nvert = 0
      nface(0) = 0
      nface(1) = 0
c
      write (uout,800)
c
c.....Read and interpret the next input file order:
c
      done = .false.
      do while (.not. done)
         ok = finputline (uin, line)
         if (.not. ok) call error ('parametric',
     &                 'Error looking for next input line', faterr)
         lp = 1
         word = getword (word, line, lp)
         word = uppcase (word)
         lw = leng(word)
c
c.......................................................................
         if (word(1:lw) .eq. 'CUBICATE') then
c.......................................................................
c
            ok = setint (divlevel, line, lp)
            phaseth = 0d0
            phaseph = 0d0
            title = 'Sucessive cubication of the sphere'
            call cubicate (divlevel, phaseth, phaseph, uout)
c
c.......................................................................
         else if (word(1:lw) .eq. 'TRIANGULATE') then
c.......................................................................
c
            ok = setint (divlevel, line, lp)
            phaseth = 0d0
            phaseph = 0d0
            title = 'Sucessive triangulation of the sphere'
            call triangulate (divlevel, phaseth, phaseph, uout)
c
c.......................................................................
         else if (word(1:lw) .eq. 'GEODESIC') then
c.......................................................................
c
            ok = setint (npts, line, lp)
            phaseth = 0d0
            phaseph = 0d0
            title = 'Geodesic (meridian & parallels) tessellation'
            call geodesic (npts, phaseth, phaseph, uout)
c
c.......................................................................
         else if (word(1:lw) .eq. 'RANDOM') then
c.......................................................................
c
            ok = setint (npts, line, lp)
            title = 'Tessel starts with a random distribution of points'
            call randomstart (npts, uout)
c
c.......................................................................
         else if (word(1:lw) .eq. 'READPOINTS') then
c.......................................................................
c
            datafile = getword (datafile, line, lp)
            title = 'Initial distribution of points is read from a file'
            call readpoints (datafile, uout)
c
c.......................................................................
         else if (word(1:lw) .eq. 'BASINPLOT') then
c.......................................................................
c
            call readbasin (line, lp, uout)
c
c.......................................................................
         else if (word(1:lw) .eq. 'BASINDENSITY') then
c.......................................................................
c
            ok = setdble (isodensity, line, lp)
            datafile = getword (datafile, line, lp)
            call readdbasin (isodensity, datafile, uout)
c
c.......................................................................
         else if (word(1:lw) .eq. 'READOFF') then
c.......................................................................
c
            if (leng(line) .gt. lp) fileoff = getword (word, line, lp)
            call readoff (fileoff, title, uout)
c
c.......................................................................
         else if (word(1:lw) .eq. 'POVRAY') then
c.......................................................................
c
c...........Select output of a povray file:
c
            dopov = .true.
            if (leng(line) .gt. lp) filepov = getword (word, line, lp)
            call getnormals (ndeg, uout)
            if (ndeg .gt. 0) then
               call removedeg (ndeg, uout)
               call getnormals (ndeg, uout)
            endif
            call writepov (filepov, title, uout)
c
c.......................................................................
         else if (word(1:lw) .eq. 'OFF') then
c.......................................................................
c
c...........Select output of an OFF file:
c
            dooff = .true.
            if (leng(line) .gt. lp) fileoff = getword (word, line, lp)
            call writeoff (fileoff, title, uout)
c
c.......................................................................
         else if (word(1:lw) .eq. 'VRML') then
c.......................................................................
c
c...........Select output of an VRML file:
c
            dovrml = .true.
            if (leng(line) .gt. lp) filevrml = getword (word, line, lp)
            call writevrml1 (filevrml, title, uout)
c
c.......................................................................
         else if (word(1:lw) .eq. 'WRITEPOINTS') then
c.......................................................................
c
            datafile = getword (datafile, line, lp)
            call writepoints (datafile, title, uout)
c
c.......................................................................
         else if (word(1:lw) .eq. 'OPTIMIZE') then
c.......................................................................
c
            ok = setint (msteps, line, lp)
            ok = ok .and. setdble (facini, line, lp)
            call steepest (uout, facini, msteps)
c
c.......................................................................
         else if (word(1:lw) .eq. '2DPLOT') then
c.......................................................................
c
            word = getword (word, line, lp)
            word = uppcase (word)
            lw = leng(word)
            if (word(1:lw) .eq. 'DBASIN') then
               ok = setdble (hoffset, line, lp)
               ok = ok .and. setdble (voffset, line, lp)
               datafile = getword (datafile, line, lp)
               ok = ok .and. leng(datafile).gt.0
               fileoff = getword (fileoff, line, lp)
               ok = ok .and. leng(fileoff).gt.0
               if (ok) call wr2ddbasin (hoffset, voffset, datafile,
     &                                  fileoff, uout)
            else
               ok = .false.
            endif
            if (.not.ok) call error ('parametric', 'Correct syntaxis:'//
     &        '2DPLOT DBASIN hoffset voffset dbasinfile gnufile',
     &        warning)
c
c.......................................................................
         else if (word(1:lw) .eq. 'SET') then
c.......................................................................
c
            call setvariables (line, lp, uout)
c
c.......................................................................
         else if (word(1:1) .eq. '#') then
c.......................................................................
c
            continue
c
c.......................................................................
         else if (word(1:lw) .eq. 'ENDTESSELATION') then
c.......................................................................
c
            done = .true.
c
c.......................................................................
         else
c.......................................................................
            call error ('dotessel' , 'order not recognized', faterr)
c
         endif
      enddo
c
c.....FORMATS:
c
 800  format (/
     & 1x, 'DOTESSEL: Tesselation of an unit sphere')
c
      end
