c
c-----------------------------------------------------------------------
c
      subroutine geodesic (npoints, phaseth, phaseph, uout)
c
c.....geodesic - tessellation of the sphere by means of N parallels
c     and N meridians, where N is the largest integer such that
c     N*N<=NPOINTS.
c
      include           'implicit.inc'
c
      integer           npoints, uout
      real*8            phaseth, phaseph
c
c.....Polyhedra data structure:
c
      include           'polyhedron.inc'
c
      integer           i, j, i1, j1, n, nf
      real*8            PI, PI2, theta, phi, stheta
c
      integer           vind
      vind(i,j) = (i-1)*n + j
c
      n = int(sqrt(dble(npoints)))
      nvert = n*n
c
c.....Get the coordinates of all points. North and South poles will be
c     repeated n times each:
c
      PI = 4d0 * atan(1d0)
      PI2 = PI + PI
      i1 = 0
      do i = 1, n
         theta = (i-1)*PI/(n-1)
         stheta = sin(theta)
         do j = 1, n
            i1 = i1 + 1
            phi = (j-1)*PI2/(n-1)
            sthcph(i1) = stheta * cos(phi)
            sthsph(i1) = stheta * sin(phi)
            cth(i1) = cos(theta)
            xvert(i1) = radius * sthcph(i1)
            yvert(i1) = radius * sthsph(i1)
            zvert(i1) = radius * cth(i1)
         enddo
      enddo
c
c.....Store the faces:
c
      nvf = 4         !Each face will be a quadrilateral
      isw1 = 0
      nf = 0
c
c.....First the triangular faces containing the north pole:
c
      do j = 1, n
         j1 = j + 1
         if (j1.gt.n) j1 = 1
         nf = nf + 1
         iface(1,nf,0) = vind(1,1)
         iface(2,nf,0) = vind(2,j)
         iface(3,nf,0) = vind(2,j1)
         iface(4,nf,0) = vind(1,1)
      enddo
c
c.....Second the triangular faces containing the south pole:
c
      do j = n, 1, -1
         j1 = j - 1
         if (j1.lt.1) j1 = n
         nf = nf + 1
         iface(1,nf,0) = vind(n,n)
         iface(2,nf,0) = vind(n-1,j)
         iface(3,nf,0) = vind(n-1,j1)
         iface(4,nf,0) = vind(n,n)
      enddo
c
c.....Finally the rest of quadrilateral faces:
c
      do i = 2, n-2
         do j = 1, n
            j1 = j + 1
            if (j1.gt.n) j1 = 1
            nf = nf + 1
            iface(1,nf,0) = vind(i,j)
            iface(2,nf,0) = vind(i+1,j)
            iface(3,nf,0) = vind(i+1,j1)
            iface(4,nf,0) = vind(i,j1)
         enddo
      enddo
c
      nface(0) = nf
c
      write (uout,100) phaseth, phaseph
      write (uout,105) nvert, nface(isw1)
 100  format (/
     & 1x, 'GEODESIC: Tesselation of the sphere by means of a linear'/
     & 1x, '          grid for meridians and paralleles.'//
     & 1x, 'Phase shift for theta coordinate: ', e12.4/
     & 1x, 'Phase shift for phi   coordinate: ', e12.4 )
 105  format (
     & 1x, 'Number of vertices created:    ', i6/
     & 1x, 'Number of quadrilateral faces: ', i6)
c
      end
