c
c-----------------------------------------------------------------------
c
      subroutine getnormals (ndegenerate, uout)
c
c.....getnormals - determine normal vectors of all the faces and
c     average normals for each vertex.
c
      include           'implicit.inc'
c
c.....Polyhedra data structure:
c
      include           'polyhedron.inc'
      include           'error.inc'
      include           'molge.inc'
      include           'normals.inc'
c
      integer           ndegenerate
      integer           uout
c
      real*8            EPSILON
      parameter         (EPSILON = 1d-20)
c
      integer           i, i1, i2, i3, nn, nfacesinvertex(mvert)
      real*8            x12, y12, z12, x13, y13, z13, aa, bb, cc, xn
c
c.....The polyhedron is assumed to be made of clockwise triangles.
c     If not, it will be reconstructed as a sum of triangles:
c
      ndegenerate = 0
      if (nvert .le. 0) return
      if (nvf .eq. 3) then
         continue
      else if (nvf .eq. 4) then
         nn = nface(isw1)
         do i = 1, nface(isw1)
            nn = nn + 1
            iface(1,nn,isw1) = iface(3,i,isw1)
            iface(2,nn,isw1) = iface(4,i,isw1)
            iface(3,nn,isw1) = iface(1,i,isw1)
         enddo
         nface(isw1) = nn
         nvf = 3
      else
         call error ('getnormals', 'faces can only be traingles or'//
     &               ' quadrilaterals', faterr)
      endif
c
c.....Initialize the vertices normals:
c
      do i = 1, nvert
         vnormal(1,i) = 0d0
         vnormal(2,i) = 0d0
         vnormal(3,i) = 0d0
         nfacesinvertex(i) = 0
      enddo
c
c.....Determine the normal vector for each triangular face:
c
      ndegenerate = 0
      do i = 1, nface(isw1)
         i1 = iface(1,i,isw1)
         i2 = iface(2,i,isw1)
         i3 = iface(3,i,isw1)
         x12 = xvert(i1) - xvert(i2)
         y12 = yvert(i1) - yvert(i2)
         z12 = zvert(i1) - zvert(i2)
         x13 = xvert(i1) - xvert(i3)
         y13 = yvert(i1) - yvert(i3)
         z13 = zvert(i1) - zvert(i3)
         aa = (y12*z13 - z12*y13)
         bb = (z12*x13 - x12*z13)
         cc = (x12*y13 - y12*x13)
         xn = sqrt(aa*aa + bb*bb + cc*cc)
c
c........Dirty trick to deal with degenerate faces: Just ignore
c        their contribution to the normal:
c
         if (xn .ge. EPSILON) then
            xn = 1d0 / xn
            aa = aa * xn
            bb = bb * xn
            cc = cc * xn
            vnormal(1,i1) = vnormal(1,i1) + aa
            vnormal(2,i1) = vnormal(2,i1) + bb
            vnormal(3,i1) = vnormal(3,i1) + cc
            nfacesinvertex(i1) = nfacesinvertex(i1) + 1
            vnormal(1,i2) = vnormal(1,i2) + aa
            vnormal(2,i2) = vnormal(2,i2) + bb
            vnormal(3,i2) = vnormal(3,i2) + cc
            nfacesinvertex(i2) = nfacesinvertex(i2) + 1
            vnormal(1,i3) = vnormal(1,i3) + aa
            vnormal(2,i3) = vnormal(2,i3) + bb
            vnormal(3,i3) = vnormal(3,i3) + cc
            nfacesinvertex(i3) = nfacesinvertex(i3) + 1
         else
            ndegenerate = ndegenerate + 1
         endif
      enddo
c
      do i = 1, nvert
         if (nfacesinvertex(i) .gt. 0) then
            vnormal(1,i) = vnormal(1,i) / nfacesinvertex(i)
            vnormal(2,i) = vnormal(2,i) / nfacesinvertex(i)
            vnormal(3,i) = vnormal(3,i) / nfacesinvertex(i)
         else
            write (uout,105) i
         endif
      enddo
c
      write (uout,100) ndegenerate
 100  format (/
     & 1x, 'GETNORMALS: Determination of the normal vectors for each'/
     & 1x, ' vertex as an average of the normals for all faces to'/
     & 1x, ' which the vertex belongs.' /
     & 1x, 'Number of degenerate faces found: ', i6 )
 105  format (
     & 1x, 'Warning: Vertex', i6, ' only belongs to degenerate faces')
c
      end
