c
c-----------------------------------------------------------------------
c
      subroutine madelung (line, lp, uin, uout)
c
c.....madelung - reads and executes the order to calculate the madelung
c     potential of the crystal in a collection of points.
c
      include           'implicit.inc'
      include           'datatm.inc'
      include           'error.inc'
      include           'molec.inc'
      include           'ewald.inc'
      include           'molge.inc'
      include           'polyhedron.inc'
      include           'strtools.inc'
c
      character*(*)     line
      integer           lp, uin, uout
c
      real*8            EPSDIS
      parameter         (EPSDIS = 1d-5)
c
      character*(80)    file1
      integer           lw
      character*(80)    word
      logical           doagain, ok1, counted
      real*8            xxx, yyy, zzz, rrr, qqq, fff
      integer           npts, nerr, ierr, nsh
      real*8            phaseph, phaseth
      real*8            deg2rad, h1, h2, h3, car2crys(3,3)
c
      integer           i, j, i1, j1, nf, nptsu, nptsv, vind
      real*8            x0, y0, z0, x1, y1, z1, x2, y2, z2
      real*8            xdu, ydu, zdu, xdv, ydv, zdv
c
      integer           leng
      logical           setdble, setint
      character*(80)    uppcase, getword
      real*8            cellcharge, fewald
c
      integer           cook_linear, cook_atan, cook_tanh
      parameter         (cook_linear = 1, cook_atan = 2, cook_tanh = 3)
      real*8            cookfactor
      integer           cook_method
c
      vind(i,j) = (i-1)*nptsv + j
c
      write (uout,800)
c
      color_mode = color_full
      cook_method = cook_atan
      cookfactor = 1d0
      word = uppcase (getword (word, line, lp))
      lw = leng(word)
      doagain = lw .gt. 0
c
c.......................................................................
      do while (doagain)
c.......................................................................
c
c.......................................................................
         if (word(1:1) .eq. '#') then
c.......................................................................
c
            doagain = .false.
c
c.......................................................................
         else if (word(1:lw) .eq. 'READIN') then
c.......................................................................
c
            word = uppcase (getword (word, line, lp))
            lw = leng(word)
            file1 = getword (file1, line, lp)
            if (lw .lt. 1) then
               call error ('Madelung', 'error on READIN order', faterr)
            else if (word(1:lw) .eq. 'OFF') then
               call readoff (file1, 'any title', uout)
            else if (word(1:lw) .eq. 'POINTS') then
               call readpoints (file1, uout)
            else if (word(1:lw) .eq. 'BASIN') then
               call readbb (file1, uout, 1)
            else if (word(1:lw) .eq. 'DBASIN') then
               call readbb (file1, uout, 2)
            else
               call error ('Madelung', 'error on READIN order', faterr)
            endif
c
c.......................................................................
         else if (word(1:lw) .eq. 'GEODESIC') then
c.......................................................................
c
            ok1 = setint (npts, line, lp)
            ok1 = ok1 .and. setdble (xxx, line, lp)
            ok1 = ok1 .and. setdble (yyy, line, lp)
            ok1 = ok1 .and. setdble (zzz, line, lp)
            ok1 = ok1 .and. setdble (rrr, line, lp)
c
c...........convert the crystallographic (xxx, yyy, zzz) coordinates:
c
            xxx = xxx*crys2car(1) + yyy*crys2car(2) + zzz*crys2car(3)
            yyy =                   yyy*crys2car(4) + zzz*crys2car(5)
            zzz =                                     zzz*crys2car(6)
c
c...........use the geodesic routine to get the sampling points:
c
            if (ok1) then
               phaseph = 0d0
               phaseth = 0d0
               call geodesic (npts, phaseth, phaseph, uout)
               do i = 1, nvert
                  xvert(i) = xxx + rrr * sthcph(i)
                  yvert(i) = yyy + rrr * sthsph(i)
                  zvert(i) = zzz + rrr * cth(i)
               enddo
            endif
c
c.......................................................................
         else if (word(1:lw) .eq. 'PLANE') then
c.......................................................................
c
            ok1 = setdble (x0, line, lp)
            ok1 = ok1 .and. setdble (y0, line, lp)
            ok1 = ok1 .and. setdble (z0, line, lp)
            ok1 = ok1 .and. setdble (x1, line, lp)
            ok1 = ok1 .and. setdble (y1, line, lp)
            ok1 = ok1 .and. setdble (z1, line, lp)
            ok1 = ok1 .and. setdble (x2, line, lp)
            ok1 = ok1 .and. setdble (y2, line, lp)
            ok1 = ok1 .and. setdble (z2, line, lp)
            ok1 = ok1 .and. setint (nptsu, line, lp)
            ok1 = ok1 .and. setint (nptsv, line, lp)
            if (ok1) then
c
c..............convert the crystallographic coordinates:
c
               x0 = x0*crys2car(1) + y0*crys2car(2) + z0*crys2car(3)
               y0 =                  y0*crys2car(4) + z0*crys2car(5)
               z0 =                                   z0*crys2car(6)
               x1 = x1*crys2car(1) + y1*crys2car(2) + z1*crys2car(3)
               y1 =                  y1*crys2car(4) + z1*crys2car(5)
               z1 =                                   z1*crys2car(6)
               x2 = x2*crys2car(1) + y2*crys2car(2) + z2*crys2car(3)
               y2 =                  y2*crys2car(4) + z2*crys2car(5)
               z2 =                                   z2*crys2car(6)
c
c..............determine the plane grid lenght sides:
c
               xdu = (x1 - x0) / (nptsu - 1)
               ydu = (y1 - y0) / (nptsu - 1)
               zdu = (z1 - z0) / (nptsu - 1)
               xdv = (x2 - x0) / (nptsv - 1)
               ydv = (y2 - y0) / (nptsv - 1)
               zdv = (z2 - z0) / (nptsv - 1)
               nvert = nptsu * nptsv
               do i = 1, nptsu
                  do j = 1, nptsv
                     i1 = vind(i,j)
                     xvert(i1) = x0 + (i-1) * xdu + (j-1) * xdv
                     yvert(i1) = y0 + (i-1) * ydu + (j-1) * ydv
                     zvert(i1) = z0 + (i-1) * zdu + (j-1) * zdv
                  enddo
               enddo
c
c..............determine the plane polygons:
c
               nvf = 4     !Each face will be a quadrilateral
               isw1 = 0
               nf = 0
               do i = 1, nptsu-1
                  i1 = i + 1
                  do j = 1, nptsv-1
                     j1 = j + 1
                     nf = nf + 1
                     iface(1,nf,0) = vind(i,j)
                     iface(2,nf,0) = vind(i1,j)
                     iface(3,nf,0) = vind(i1,j1)
                     iface(4,nf,0) = vind(i,j1)
                  enddo
               enddo
               nface(0) = nf
            endif
c
c.......................................................................
         else if (word(1:lw) .eq. 'COLORMODE') then
c.......................................................................
c
            word = uppcase (getword (word, line, lp))
            lw = leng(word)
            if (word(1:lw) .eq. 'BINARY') then
               color_mode = color_binary
            else if (word(1:lw) .eq. 'GREY') then
               color_mode = color_grey
            else if (word(1:lw) .eq. 'FULL') then
               color_mode = color_full
            else
               color_mode = color_full
               call error ('Madelung',
     &              'Unknown color mode. Using FULL instead', warning)
            endif
c
c.......................................................................
         else if (word(1:lw) .eq. 'COLORSCALE') then
c.......................................................................
c
            word = uppcase (getword (word, line, lp))
            lw = leng(word)
            if (word(1:lw) .eq. 'LINEAR') then
               cook_method = cook_linear
            else if (word(1:lw) .eq. 'ATAN') then
               cook_method = cook_atan
               cookfactor = 1d0
            else if (word(1:lw) .eq. 'ATANF') then
               cook_method = cook_atan
               ok1 = setdble (cookfactor, line, lp)
               if (.not. ok1) then
                  call error ('Madelung',
     &               'Wrong atan factor. Using default: 1.0', warning)
                  cookfactor = 1d0
               endif
            else if (word(1:lw) .eq. 'TANH') then
               cook_method = cook_tanh
               cookfactor = 1d0
            else if (word(1:lw) .eq. 'TANHF') then
               cook_method = cook_tanh
               ok1 = setdble (cookfactor, line, lp)
               if (.not. ok1) then
                  call error ('Madelung',
     &               'Wrong tanh factor. Using default: 1.0', warning)
                  cookfactor = 1d0
               endif
            else
               cook_method = cook_atan
               cookfactor = 1d0
               call error ('Madelung',
     &            'Unknown color scale. Using default: DOATAN', warning)
            endif
c
c.......................................................................
         endif
c.......................................................................
         if (doagain) then
            word = uppcase (getword (word, line, lp))
            lw = leng(word)
            doagain = lw .gt. 0
         endif
c.......................................................................
      enddo
c.......................................................................
c
c.....obtain the matrix to transform cartesian to crystal coordinates:
c
      deg2rad = atan(1d0) / 45d0
      h1 = sqrt(1d0-cgamm*cgamm)
      h2 = (calph-cbeta*cgamm) / sqrt(1d0-cgamm*cgamm )
      h3 = sqrt(1-cbeta*cbeta-(calph-cbeta*cgamm)**2/(1-cgamm*cgamm))
      car2crys(1,1) = 1d0 / ared
      car2crys(2,1) = 0d0
      car2crys(3,1) = 0d0
      car2crys(1,2) = -cgamm / (ared * h1)
      car2crys(2,2) = 1d0 / (bred * h1)
      car2crys(3,2) = 0d0
      car2crys(1,3) = ((h2/h1) * cgamm - cbeta) / (ared * h3)
      car2crys(2,3) = -h2 / (bred * h1 * h3)
      car2crys(3,3) = 1d0 / (cred * h3)
c
c.....Prepare some Ewald data:
c
      call preewald ()
c
c.....No special cluster affects the pure crystal potential:
c
      nmod = 0
c
c.....Discount hole positions if occupied by a charged ion in the pure
c     crystal:
c
      do i = 1, nhole
         counted = .false.
         j = 1
         do while (j.le.nmod .and. .not. counted)
            counted = abs(xhole(i)-xmod(j)) .le. EPSDIS .and.
     &                abs(yhole(i)-ymod(j)) .le. EPSDIS .and.
     &                abs(zhole(i)-zmod(j)) .le. EPSDIS
            j = j + 1
         enddo
         if (.not. counted) then
            qqq = cellcharge (xhole(i), yhole(i), zhole(i))
            if (abs(qqq) .gt. 1d-6) then
               nmod = nmod + 1
               xmod(nmod) = xhole(i)
               ymod(nmod) = yhole(i)
               zmod(nmod) = zhole(i)
               qmod(nmod,1) = 0d0
               qmod(nmod,2) = qqq
            endif
         endif
      enddo
c
c.....Add charged cluster positions:
c
      do i = 1, nclus
         qqq = qclus(typclus(i))
         if (abs(qqq) .gt. 1d-6) then
            counted = .false.
            j = 0
            do while (j.lt.nmod .and. .not. counted)
               j = j + 1
               counted = abs(xclus(i)-xmod(j)) .le. EPSDIS .and.
     &                   abs(yclus(i)-ymod(j)) .le. EPSDIS .and.
     &                   abs(zclus(i)-zmod(j)) .le. EPSDIS
            enddo
            if (counted) then
               qmod(j,1) = qmod(j,1) + qqq
            else
               nmod = nmod + 1
               xmod(nmod) = xclus(i)
               ymod(nmod) = yclus(i)
               zmod(nmod) = zclus(i)
               qmod(nmod,1) = qqq
               qmod(nmod,2) = 0d0
            endif
         endif
      enddo
c
c.....Keep the number of errors in caling Ewald:
c
      nerr = 0
c
c.....Run over the vertex of the polyhedron:
c
*mdc*if debug
      write (uout,900)
 900  format (/
     & 1x, 'Madelung potential:'/
     & 1x, '     cartesian      crystallographic'/
     & 1x, '----x---- ----y---- ----z----',
     & 1x, '----x---- ----y---- ----z---- -charge- ----ewald----')
 905  format (6f10.5, f9.4, 1p, 2e14.6, 0p)
*mdc*endif
      do i = 1, nvert
c
c........Convert the cartesian into crystallographic coordinates:
c
         xxx = car2crys(1,1)*xvert(i) + car2crys(1,2)*yvert(i) +
     &         car2crys(1,3)*zvert(i)
         yyy = car2crys(2,1)*xvert(i) + car2crys(2,2)*yvert(i) +
     &         car2crys(2,3)*zvert(i)
         zzz = car2crys(3,1)*xvert(i) + car2crys(3,2)*yvert(i) +
     &         car2crys(3,3)*zvert(i)
c
c........Is there any ion on this position:
c
         qqq = cellcharge (xxx, yyy, zzz)
c
c........Get and store the value of the Madelung potential:
c
         fff = fewald (xxx, yyy, zzz, qqq, nsh, ierr)
         if (ierr .ne. 0) nerr = nerr + 1
         if (cook_method .eq. cook_linear) then
            fun_value(i) = fff
         else if (cook_method .eq. cook_tanh) then
            fun_value(i) = tanh(cookfactor * fff)
         else
            fun_value(i) = atan(cookfactor * fff)
         endif
*mdc*if debug
         write (uout,905) xvert(i), yvert(i), zvert(i), xxx, yyy, zzz,
     &                    qqq, fff, fun_value(i)
*mdc*endif
      enddo
c
      write (uout,810) nvert, nerr
c
c.....FORMATS:
c
 800  format (/
     & 1x, 'MADELUNG(start): Computing the crystal Madelung potential'/
     & 1x, 'in a grid of points.'/
     & 1x, '(NOTICE: the probe charge is **ONE ELECTRON**, -e)')
 810  format (/
     & 1x, 'MADELUNG(end):'/
     & 1x, 'Number of sampling points: ', i5/
     & 1x, 'Number of errors in Ewald: ', i5)
c
      end
