c
c-----------------------------------------------------------------------
c
      subroutine molbndgeom (uout)
c
c.....molbndgeom - determine the bond geometry (distances and angles)
c     for the nonequivalent ions in the molecular model.
c
      include           'implicit.inc'
      include           'consts.inc'
      include           'datatm.inc'
      include           'error.inc'
      include           'molec.inc'
c
      integer           uout
c
      integer           MBND
      parameter         (MBND = MMOL)
      integer           nbnd, ibnd(MBND), iord(MBND)
      real*8            theta(MBND), dist(MBND)
c
      integer           i, i1, j, j1, k, k1, ln
      real*8            defdmin, defdmax, xdum
      real*8            xx, yy, zz, d2
      real*8            x10, y10, z10, r10, x20, y20, z20, r20, r12
      real*8            xi, yi, zi, ri, rj
      integer           qi, atmZi, qj, atmZj
c
      integer           leng
c
      defdmin = 1d-4
      write (uout,600)
c
c.....Analyze the bonds of the non-equivalent pure crystal ions:
c
      do i = 1, nneq
c
c........Retrieve the ion properties, including radius:
c
         qi = nint(qneq(typmol(i)))
         atmZi = zzneq(typmol(i))
         if (typrad .eq. typION) then
            ri = (atmION(qi,atmZi)/bohr2pm) * ionfactor(qi)
         else if (typrad .eq. typvdW) then
            ri = (atmvdW(atmZi)/bohr2pm) * ionfactor(qi)
         else if (typrad .eq. typcov) then
            ri = (atmcov(atmZi)/bohr2pm) * ionfactor(qi)
         endif
*mdc*if method2
*c
*c........Search for an ion of this type in the molecular model:
*c
*         i1 = 1
*         do while (codemol(i1).ne.codecrystal .or. typmol(i1).ne.i)
*            i1 = i1 + 1
*            if (i1 .gt. nmol) then
*               write (uout,605) i, namneq(i)(1:leng(namneq(i)))
*               goto 10
*            endif
*         enddo
*         xi = xmol(i1)
*         yi = ymol(i1)
*         zi = zmol(i1)
*mdc*else
         xi = xneq(i)
         yi = yneq(i)
         zi = zneq(i)
*mdc*endif
         write (uout,610) i, namneq(i)(1:leng(namneq(i))), xi, yi, zi
c
c........Examine the distance to the atoms in the molecular model:
c
         nbnd = 0
         do j = 1, nmol
c
c...........Retrieve the ion properties, including radius:
c
            if (codemol(j).eq.codecrystal) then
               qj = nint(qneq(typmol(j)))
               atmZj = zzneq(typmol(j))
            else
               qj = nint(qclus(typmol(j)))
               atmZj = zzclus(typmol(j))
            endif
            if (typrad .eq. typION) then
               rj = (atmION(qj,atmZj)/bohr2pm) * ionfactor(qj)
            else if (typrad .eq. typvdW) then
               rj = (atmvdW(atmZj)/bohr2pm) * ionfactor(qj)
            else if (typrad .eq. typcov) then
               rj = (atmcov(atmZj)/bohr2pm) * ionfactor(qj)
            endif
            xx = (xi-xmol(j)) * ared
            yy = (yi-ymol(j)) * bred
            zz = (zi-zmol(j)) * cred
            d2 = xx*xx + yy*yy + zz*zz + 2d0 *
     &           (xx*yy*cgamm + yy*zz*calph + zz*xx*cbeta)
            d2 = sqrt(d2)
            defdmax = ri + rj
            if (d2.gt.defdmin .and. d2.le.defdmax) then
c
c..............Print the ions bonded to the central ion:
c
               nbnd = nbnd + 1
               ibnd(nbnd) = j
               iord(nbnd) = nbnd
               dist(nbnd) = d2
            endif
         enddo
c
c........Sort the neighbors by the bond distance:
c
         call qcksort (dist, iord, 1, nbnd)
c
c........Print out the bond distances:
c
         do j = 1, nbnd
            j1 = ibnd(iord(j))
            if (codemol(j1) .eq. codecrystal) then
               ln = leng(namneq(typmol(j1)))
               write (uout,615) j, j1, xmol(j1), ymol(j1), zmol(j1),
     &               typmol(j1), namneq(typmol(j1))(1:ln),
     &               dist(iord(j))
            else
               ln = leng(namclus(typmol(j)))
               write (uout,616) j, j1, xmol(j1), ymol(j1), zmol(j1),
     &               typmol(j1), namclus(typmol(j1))(1:ln),
     &               dist(iord(j))
            endif
         enddo
c
c........Print now the bond angles:
c
         write (uout,620)
         do j = 1, nbnd
            j1 = ibnd(iord(j))
            x10 = xi - xmol(j1)
            y10 = yi - ymol(j1)
            z10 = zi - zmol(j1)
            r10 = sqrt(x10*(x10*g(1) + 2d0*y10*g(2) + 2d0*z10*g(4))+
     &                 y10*(y10*g(3) + 2d0*z10*g(5)) + z10*z10*g(6))
            do k = 1, j-1
               k1 = ibnd(iord(k))
               x20 = xi - xmol(k1)
               y20 = yi - ymol(k1)
               z20 = zi - zmol(k1)
               r20 = sqrt(x20*(x20*g(1) + 2d0*y20*g(2) + 2d0*z20*g(4))+
     &                    y20*(y20*g(3) + 2d0*z20*g(5)) + z20*z20*g(6))
               r12 = g(1)*x10*x20 + g(2)*(y10*x20+x10*y20) +
     &               g(3)*y10*y20 + g(4)*(z10*x20+x10*z20) +
     &               g(5)*(z10*y20+y10*z20) + g(6)*z10*z20
               xdum = r12 / (r10*r20)
               if (abs(xdum) .gt. 1d0) then
                  xdum = max(-1d0, min(+1d0, xdum))
*mdc*if debug
*                  write (uout,*) 'DEBUG(molbndgeom): bond angle error',
*     &                  ' (ijk) (r10,r20,r12,xdum) -> ',i, j, k,
*     &                  r10, r20, r12, xdum
*mdc*endif
               endif
               theta(k) = acos(xdum) / rad
            enddo
            if (j.gt.1) write (uout,625) (theta(k), k = 1, j-1)
         enddo
 10      continue
      enddo
c
 600  format (/
     & 1x, 'MOLBNDGEOM: Analysis of the bond distances and angles')
 605  format (
     & 1x, 'The molecular model does not contain pure crystal atoms of',
     &     'type ', i3, 2x, a)
 610  format (/
     & 1x, 'Bonds of the NEQ pure crystal ion #', i2, 2x, a/
     & 1x, 'Position: ', 3f12.5/
     & 1x, 'bond-neig----------------Position------------',
     &     '---------type--------distance--')
 615  format (
     & 2i5, 3f12.5, 1x, 'crystal', i3, 1x, a6, 1p, e13.5, 0p)
 616  format (
     & 2i5, 3f12.5, 1x, 'cluster', i3, 1x, a6, 1p, e13.5, 0p)
 620  format (/
     & 1x, 'Bond angles (degrees):')
 625  format (1x, 10f7.2)
c
      end
