c
c-----------------------------------------------------------------------
c
      subroutine molbuildcell (uout)
c
c.....molbuildcell - builds the molecular system.
c     The system is made as:  crystal - hole + cluster.
c
      include           'implicit.inc'
      include           'consts.inc'
      include           'error.inc'
      include           'molec.inc'
      include           'molge.inc'
      include           'spgcom.inc'
c
      integer           uout
c
c LOCAL VARIABLES:
c
      real*8            EPSDIS
      parameter         (EPSDIS = 1d-5)
      real*8            xx(MCELL), yy(MCELL), zz(MCELL)
      real*8            sc, omega, xl, yl, zl, deg2rad
      real*8            gtensor(3,3), grtensor(3,3)
      integer           i, j, k, ki, l, m, n, ncellant, natact, mrot
      logical           done, ishole, repeated, enlarged
c
      write (uout,800)
c
c.....check the molecular and graphics clipping:
c
      enlarged = x0molclip.gt.x0grfclip .or. x1molclip.lt.x1grfclip
     &      .or. y0molclip.gt.y0grfclip .or. y1molclip.lt.y1grfclip
     &      .or. z0molclip.gt.z0grfclip .or. z1molclip.lt.z1grfclip
      if (enlarged) then
         x0molclip = min (x0grfclip, x0molclip)
         x1molclip = max (x1grfclip, x1molclip)
         y0molclip = min (y0grfclip, y0molclip)
         y1molclip = max (y1grfclip, y1molclip)
         z0molclip = min (z0grfclip, z0molclip)
         z1molclip = max (z1grfclip, z1molclip)
         write (uout,810) x0molclip, x1molclip, y0molclip, y1molclip,
     &                    z0molclip, z1molclip 
      endif
c
      deg2rad = atan(1d0) / 45d0
c
c.....obtain the metric tensor:
c
      g(1) = ared * ared
      g(2) = ared * bred * cgamm
      g(3) = bred * bred
      g(4) = ared * cred * cbeta
      g(5) = bred * cred * calph
      g(6) = cred * cred
      gtensor(1,1) = g(1)
      gtensor(1,2) = g(2)
      gtensor(2,1) = g(2)
      gtensor(2,2) = g(3)
      gtensor(1,3) = g(4)
      gtensor(3,1) = g(4)
      gtensor(2,3) = g(5)
      gtensor(3,2) = g(5)
      gtensor(3,3) = g(6)
c
c.....get the reciprocal lattice parameters:
c
      sc = 0.5d0 * (alph + beta + gamm) * deg2rad
      omega = sin(sc) * sin(sc-alph*deg2rad) * sin(sc-beta*deg2rad) *
     &        sin(sc-gamm*deg2rad)
      omega = sqrt(omega) * 2d0 * ared * bred * cred
      arec = bred * cred * salph / omega
      brec = cred * ared * sbeta / omega
      crec = ared * bred * sgamm / omega
      calphr = (cbeta * cgamm - calph) / (sbeta * sgamm)
      cbetar = (cgamm * calph - cbeta) / (sgamm * salph)
      cgammr = (calph * cbeta - cgamm) / (salph * sbeta)
c
c.....computing the metric tensor of the reciprocal lattice:
c
      gr(1) = arec * arec
      gr(2) = arec * brec * cgammr
      gr(3) = brec * brec
      gr(4) = arec * crec * cbetar
      gr(5) = brec * crec * calphr
      gr(6) = crec * crec
      grtensor(1,1) = gr(1)
      grtensor(1,2) = gr(2)
      grtensor(2,1) = gr(2)
      grtensor(2,2) = gr(3)
      grtensor(1,3) = gr(4)
      grtensor(3,1) = gr(4)
      grtensor(2,3) = gr(5)
      grtensor(3,2) = gr(5)
      grtensor(3,3) = gr(6)
c
c.....Get the tensors for the primitive (real and reciprocal) lattices:
c
      do i = 1, 3
         do j = 1, i
            grp(i,j) = 0d0
            gp(i,j) = 0d0
            do ki = 1, 3
               do l = 1, 3
                  grp(i,j) = grp(i,j) + grtensor(ki,l) *
     &               tocrec(lcent,i,ki) * tocrec(lcent,j,l)
                  gp(i,j)  = gp(i,j) + gtensor(ki,l) *
     &               tocreal(lcent,ki,i) * tocreal(lcent,l,j)
               enddo
            enddo
            grp(j,i) = grp(i,j)
            gp(j,i) = gp(i,j)
         enddo
       enddo
c
c.....obtain the matrix to transform crystal to cartesian coordinates:
c
c     The matrix transforms the crystallographic coordinates of the
c     graphical elements to an orthogonal cartesian system such that:
c     a) the new x axis is parallel to the crystallographic a axis,
c     b) the xy plane is equal to the crystallographic ab plane,
c     c) the new x,y,z unit vectors are of length 1.
c
c     Use the matrix as:
c
c     do i = 1, gnpt
c        gxpt(i) = gxpt(i)*crys2car(1) + gypt(i)*crys2car(2) +
c    &             gzpt(i)*crys2car(3)
c        gypt(i) = gypt(i)*crys2car(4) + gzpt(i)*crys2car(5)
c        gzpt(i) = gzpt(i)*crys2car(6)
c     enddo
c
      crys2car(1) = ared
      crys2car(2) = bred * cgamm
      crys2car(3) = cred * cbeta
      crys2car(4) = bred * sqrt(1-cgamm*cgamm)
      crys2car(5) = cred * (calph-cbeta*cgamm) / sqrt(1-cgamm*cgamm)
      crys2car(6) = cred *
     &    sqrt(1-cbeta*cbeta-(calph-cbeta*cgamm)**2/(1-cgamm*cgamm))
c
c.....translate all crystal positions to the unit cell:
c
      do i = 1, nneq
         if (xneq(i) .lt. 0d0) then
            xneq(i) = xneq(i) - int(xneq(i)) + 1d0
         else
            xneq(i) = xneq(i) - int(xneq(i))
         endif
         if (yneq(i) .lt. 0d0) then
            yneq(i) = yneq(i) - int(yneq(i)) + 1d0
         else
            yneq(i) = yneq(i) - int(yneq(i))
         endif
         if (zneq(i) .lt. 0d0) then
            zneq(i) = zneq(i) - int(zneq(i)) + 1d0
         else
            zneq(i) = zneq(i) - int(zneq(i))
         endif
      enddo
c
c.......................................................................
c.....use the crystal symmetry to get all atoms in the unit cell: ......
c.......................................................................
c
      ncell = 0
      do i = 1, nneq
         ncellant = ncell
         mrot = 0
         do j = 1, neqv
            mrot = mrot + 1
            if (mrot .gt. MCELL) 
     &         call error ('create', 'mrot > MCELL!', faterr)
            xx(mrot) = rotm(1,1,j)*xneq(i) + rotm(1,2,j)*yneq(i) + 
     &                 rotm(1,3,j)*zneq(i) + rotm(1,4,j)
            yy(mrot) = rotm(2,1,j)*xneq(i) + rotm(2,2,j)*yneq(i) + 
     &                 rotm(2,3,j)*zneq(i) + rotm(2,4,j)
            zz(mrot) = rotm(3,1,j)*xneq(i) + rotm(3,2,j)*yneq(i) + 
     &                 rotm(3,3,j)*zneq(i) + rotm(3,4,j)
c
c...........Move created points to the zeroth cell
c
            if (xx(mrot) .lt. 0d0) xx(mrot) = xx(mrot)+1d0
            if (xx(mrot) .ge. 1d0) xx(mrot) = xx(mrot)-1d0
            if (yy(mrot) .lt. 0d0) yy(mrot) = yy(mrot)+1d0
            if (yy(mrot) .ge. 1d0) yy(mrot) = yy(mrot)-1d0
            if (zz(mrot) .lt. 0d0) zz(mrot) = zz(mrot)+1d0
            if (zz(mrot) .ge. 1d0) zz(mrot) = zz(mrot)-1d0
c
c...........Test for centrosymmetric lattices.
c
            if (ncent.eq.1) then
               mrot = mrot + 1
               if (mrot .gt. MCELL) 
     &            call error ('create', 'mrot > MCELL!', faterr)
               xx(mrot) = -xx(mrot-1)
               yy(mrot) = -yy(mrot-1)
               zz(mrot) = -zz(mrot-1)
               if (xx(mrot) .lt. 0d0) xx(mrot) = xx(mrot)+1d0
               if (yy(mrot) .lt. 0d0) yy(mrot) = yy(mrot)+1d0
               if (zz(mrot) .lt. 0d0) zz(mrot) = zz(mrot)+1d0
            endif
         enddo
*mdc*if debug
*         write (uout,*) 'NEQ:',i,'Before destroying pass #1'
*         write (uout,830) (j, xx(j), yy(j), zz(j), j = 1, mrot)
*mdc*endif
c
c........Destroy identical atoms.
c
         do j = 1, mrot
            repeated = .false.
            k = 1
            done = k .gt. ncell
            do while (.not. done)
               repeated = abs(xcell(k)-xx(j)).le.EPSDIS .and.
     &                    abs(ycell(k)-yy(j)).le.EPSDIS .and.
     &                    abs(zcell(k)-zz(j)).le.EPSDIS .and.
     &                    typcell(k).eq.i
               k = k + 1
               done = repeated .or. k.gt.ncell
            enddo
            if (.not. repeated) then
               ncell = ncell + 1
               xcell(ncell) = xx(j)
               ycell(ncell) = yy(j)
               zcell(ncell) = zz(j)
               typcell(ncell) = i
            endif
         enddo
*mdc*if debug
*         write (uout,*) 'NEQ:',i,'After destroying pass #1'
*         write (uout,830) (j, xcell(j), ycell(j), zcell(j), j=1,ncell)
*mdc*endif
c
c........Create centered positions.
c
         natact = ncell
         do k = 1, natact
            do j = 1, ncv
               ncell = ncell + 1
               if (ncell .gt. MCELL) 
     &            call error ('molbuildcell', 'ncell > MCELL!', faterr)
               xcell(ncell) = xcell(k) + cen(1,j)
               ycell(ncell) = ycell(k) + cen(2,j)
               zcell(ncell) = zcell(k) + cen(3,j)
               typcell(ncell) = typcell(k)
               if (xcell(ncell).ge.1d0) xcell(ncell) = xcell(ncell) - 1
               if (ycell(ncell).ge.1d0) ycell(ncell) = ycell(ncell) - 1
               if (zcell(ncell).ge.1d0) zcell(ncell) = zcell(ncell) - 1
            enddo
         enddo
c
c........Destroy identical atoms.
c
         n = 0
         do j = 1, ncell
            repeated = .false.
            k = 1
            done = k .gt. n
            do while (.not. done)
               repeated = abs(xcell(k)-xcell(j)).le.EPSDIS .and.
     &                    abs(ycell(k)-ycell(j)).le.EPSDIS .and.
     &                    abs(zcell(k)-zcell(j)).le.EPSDIS .and.
     &                    typcell(k).eq.typcell(j)
               k = k + 1
               done = repeated .or. k.gt.n
            enddo
            if (.not. repeated) then
               n = n + 1
               xcell(n) = xcell(j)
               ycell(n) = ycell(j)
               zcell(n) = zcell(j)
               typcell(n) = typcell(j)
            endif
         enddo
         ncell = n
c
c........Multiplicity of the position
c
         multneq(i) = ncell - ncellant
      enddo
c
c.......................................................................
c.....create the molecular model as: crystal - hole + cluster ..........
c.......................................................................
c
      nmol = 0
      do l = 1, ncell
         do i = int(x0molclip-1d0), int(x1molclip+1d0)
            do j = int(y0molclip-1d0), int(y1molclip+1d0)
               do k = int(z0molclip-1d0), int(z1molclip+1d0)
                  xl = xcell(l) + i
                  yl = ycell(l) + j
                  zl = zcell(l) + k
                  if (xl .ge. x0molclip .and. xl .le. x1molclip .and.
     &                yl .ge. y0molclip .and. yl .le. y1molclip .and.
     &                zl .ge. z0molclip .and. zl .le. z1molclip) then
                     ishole = .false.
                     m = 1
                     do while (.not.ishole .and. m.le.nhole)
                        ishole = abs(xl-xhole(m)).le.EPSDIS .and.
     &                           abs(yl-yhole(m)).le.EPSDIS .and.
     &                           abs(zl-zhole(m)).le.EPSDIS
                        m = m + 1
                     enddo
                     if (.not.ishole) then
                        nmol = nmol + 1
                        if (nmol .gt. MMOL) call error ('molbuildcell',
     &                      'nmol > MMOL!', faterr)
                        xmol(nmol) = xl
                        ymol(nmol) = yl
                        zmol(nmol) = zl
                        codemol(nmol) = codecrystal
                        typmol(nmol) = typcell(l)
                        inclip(nmol) =
     &                     xl.ge.x0grfclip .and. xl.le.x1grfclip .and.
     &                     yl.ge.y0grfclip .and. yl.le.y1grfclip .and.
     &                     zl.ge.z0grfclip .and. zl.le.z1grfclip
                     endif
                  endif
               enddo
            enddo
         enddo
      enddo
      do l = 1, nclus
         if (xclus(l) .ge. x0molclip .and. xclus(l) .le. x1molclip .and.
     &       yclus(l) .ge. y0molclip .and. yclus(l) .le. y1molclip .and.
     &       zclus(l) .ge. z0molclip .and. zclus(l) .le. z1molclip) then
            nmol = nmol + 1
            xmol(nmol) = xclus(l)
            ymol(nmol) = yclus(l)
            zmol(nmol) = zclus(l)
            codemol(nmol) = codecluster
            typmol(nmol) = typclus(l)
            inclip(nmol) =
     &         xclus(l).ge.x0grfclip .and. xclus(l).le.x1grfclip .and.
     &         yclus(l).ge.y0grfclip .and. yclus(l).le.y1grfclip .and.
     &         zclus(l).ge.z0grfclip .and. zclus(l).le.z1grfclip
         endif
      enddo
c
      write (uout,820) nmol
*mdc*if debug
*      write (uout,830) (i, xmol(i), ymol(i), zmol(i), i = 1, nmol)
*mdc*endif
c
c.....FORMATS:
c
 800  format (/
     & 1x, 'MOLBUILDCELL: Building the molecular system description.')
 810  format (
     & 1x, 'Crystal clipping enlarged to the graphical clipping:',
     & 1x, '(x0,x1,y0...) '/
     & 1x, 6f10.4)
 820  format (1x, 'Atoms in the molecular box: ', i6)
 830  format (
     & 1x, 'Atoms in the molecular data base:'/
     & 1x, '--i--------x--------------y--------------z--------'/
     & (i4,3f15.6))
c
      end
