c
c-----------------------------------------------------------------------
c
      subroutine molcrystal (uin, uout, title)
c
c.....molcrystal - reads in the description of the molecular system.
c     The 'molecule' is made of a crystal, a hole and a cluster.
c
      include           'implicit.inc'
      include           'consts.inc'
      include           'datatm.inc'
      include           'error.inc'
      include           'molec.inc'
      include           'molge.inc'
      include           'strtools.inc'
c
      integer           uin, uout
      character*(*)     title
c
      integer           i, ierr, zmult, nshew
      integer           lp, lw
      real*8            deg2rad, x1, x2, x3, x4, x5, x6, vol, xmass
      real*8            xcharge, vmad(MNEQ), avgcmadelung, zmin, vscale
      character*(600)   line
      character*(80)    word
      logical           doagain, ok, spgdone
      character*(3)     name
c
      integer           leng, zhlookup, mcdvec
      real*8            fewald
      logical           setdble, setint
      character*(80)    uppcase, getword
      logical           finputline
c
      write (uout,800)
c
      spg = 'p 1'
      spgdone = .false.
      ared = 1d0
      bred = 1d0
      cred = 1d0
      alph = 90d0
      beta = 90d0
      gamm = 90d0
      deg2rad = atan(1d0) / 45d0
      doagain = finputline (uin, line)
c
c.......................................................................
      do while (doagain)
c.......................................................................
c
         lp = 1
         word = uppcase (getword (word, line, lp))
         lw = leng(word)
c
c.......................................................................
         if (word(1:1) .eq. '#') then
c.......................................................................
c
            continue
c
c.......................................................................
         else if (word(1:lw) .eq. 'SPG') then
c.......................................................................
c
            do while (line(lp:lp) .eq. ' ')
               lp = lp + 1
            enddo
            spg = line(lp:leng(line))
            call spgr (spg, uout, uout, ierr)
            if (ierr .eq. 0) then
               spgdone = .true.
            else
               spg = 'p 1'
            endif
c
c.......................................................................
         else if (word(1:lw) .eq. 'CELL') then
c.......................................................................
c
            ok = setdble (ared, line, lp)
            ok = ok .and. setdble (bred, line, lp)
            ok = ok .and. setdble (cred, line, lp)
            ok = ok .and. setdble (alph, line, lp)
            ok = ok .and. setdble (beta, line, lp)
            ok = ok .and. setdble (gamm, line, lp)
            if (.not. ok) then
               call error ('molcrystal',
     &              'Input error. Crystalbox ignored', warning)
            endif
c
c.......................................................................
         else if (word(1:lw) .eq. 'NEQ') then
c.......................................................................
c
            nneq = nneq + 1
            if (nneq .gt. mneq) call error ('molcrystal',
     &        'Number of nonequivalent atoms exceeded. Increase MNEQ!',
     &        faterr)
            ok = setdble (xneq(nneq), line, lp)
            ok = ok .and. setdble (yneq(nneq), line, lp)
            ok = ok .and. setdble (zneq(nneq), line, lp)
            namneq(nneq) = getword (word, line, lp)
            ok = ok .and. leng(word).gt.0
            if (.not.ok) call error ('molcrystal',
     &         'Format error in NEQ order', faterr)
            ok = setdble (qneq(nneq), line, lp)
            if (.not.ok) qneq(nneq) = 0d0
c
c...........determining the atomic number:
c
            name = namneq(nneq)
            name = uppcase(name)
            if (name(2:2).lt.'A' .or. name(2:2).gt.'Z') then
               name(2:2) = ' '
            endif
            name(3:3) = null
            zzneq(nneq) = zhlookup (name)
            if (zzneq(nneq) .le. 0) call error ('molcrystal',
     &        'Atom not recognized in NEQ order!', faterr)
c
c.......................................................................
         else if (word(1:lw) .eq. 'HOLE') then
c.......................................................................
c
            nhole = nhole + 1
            if (nhole .gt. mhole) call error ('molcrystal',
     &        'Number of hole positions exceeded. Increase MHOLE!',
     &        faterr)
            ok = setdble (xhole(nhole), line, lp)
            ok = ok .and. setdble (yhole(nhole), line, lp)
            ok = ok .and. setdble (zhole(nhole), line, lp)
            if (.not.ok) call error ('molcrystal',
     &         'Format error in HOLE order', faterr)
c
c.......................................................................
         else if (word(1:lw) .eq. 'NEQCLUS') then
c.......................................................................
c
            nneqclus = nneqclus + 1
            if (nneqclus .gt. MNEQCLUS) call error ('molcrystal',
     &        'Too many noneq. cluster atoms. Increase MNEQCLUS!',
     &        faterr)
            ok = setint (multclus(nneqclus), line, lp)
            if (nclus+multclus(nneqclus) .gt. MCLUS)
     &        call error ('molcrystal',
     &        'Too many total cluster atoms. Increase MCLUS!', faterr)
            namclus(nneqclus) = getword (word, line, lp)
            ok = ok .and. leng(word).gt.0
            if (.not.ok) call error ('molcrystal',
     &         'Format error in NEQCLUS order', faterr)
            ok = setdble (qclus(nneqclus), line, lp)
            if (.not.ok) qclus(nneqclus) = 0d0
c
            read (uin,*) (xclus(i), yclus(i), zclus(i),
     &           i = nclus+1, nclus+multclus(nneqclus))
c
            do i = 1, multclus(nneqclus)
               typclus(nclus+i) = nneqclus
            enddo
            nclus = nclus + multclus(nneqclus)
c
c...........determining the atomic number:
c
            name = namclus(nneqclus)
            name = uppcase(name)
            if (name(2:2).lt.'A' .or. name(2:2).gt.'Z') then
               name(2:2) = ' '
            endif
            name(3:3) = null
            zzclus(nneqclus) = zhlookup (name)
            if (zzclus(nneqclus) .le. 0) call error ('molcrystal',
     &        'Atom not recognized in NEQCLUS order!', faterr)
c
c.......................................................................
         else if (word(1:lw) .eq. 'NOBOXES') then
c.......................................................................
c
            x0molclip = -1d9
            y0molclip = -1d9
            z0molclip = -1d9
            x1molclip =  1d9
            y1molclip =  1d9
            z1molclip =  1d9
c
            x0grfclip = -1d9
            y0grfclip = -1d9
            z0grfclip = -1d9
            x1grfclip =  1d9
            y1grfclip =  1d9
            z1grfclip =  1d9
c
c.......................................................................
         else if (word(1:lw) .eq. 'CRYSTALBOX') then
c.......................................................................
c
            ok = setdble (x1, line, lp)
            ok = ok .and. setdble (x2, line, lp)
            ok = ok .and. setdble (x3, line, lp)
            ok = ok .and. setdble (x4, line, lp)
            ok = ok .and. setdble (x5, line, lp)
            ok = ok .and. setdble (x6, line, lp)
            if (ok) then
               x0molclip = x1
               y0molclip = x2
               z0molclip = x3
               x1molclip = x4
               y1molclip = x5
               z1molclip = x6
            else
               call error ('molcrystal',
     &              'Input error. Crystalbox ignored', warning)
            endif
c
c.......................................................................
         else if (word(1:lw) .eq. 'CLIPPINGBOX') then
c.......................................................................
c
            ok = setdble (x1, line, lp)
            ok = ok .and. setdble (x2, line, lp)
            ok = ok .and. setdble (x3, line, lp)
            ok = ok .and. setdble (x4, line, lp)
            ok = ok .and. setdble (x5, line, lp)
            ok = ok .and. setdble (x6, line, lp)
            if (ok) then
               x0grfclip = x1
               y0grfclip = x2
               z0grfclip = x3
               x1grfclip = x4
               y1grfclip = x5
               z1grfclip = x6
            else
               call error ('molcrystal',
     &              'Input error. Clippingbox ignored', warning)
            endif
c
c.......................................................................
         else if (word(1:lw) .eq. 'ENDCRYSTAL') then
c.......................................................................
c
            doagain = .false.
c
c.......................................................................
         else if (word(1:lw) .eq. 'SET') then
c.......................................................................
c
            call setvariables (line, lp, uout)
c
c.......................................................................
         else if (word(1:lw) .eq. 'TITLE') then
c.......................................................................
c
            title = line(lp:leng(line))
c
c.......................................................................
         else
c.......................................................................
c
            call error ('molcrystal',
     &         'Input line error: order unknown'//line(1:leng(line)),
     &         warning)
c
c.......................................................................
         endif
c.......................................................................
c
c.......................................................................
         if (doagain) doagain = finputline (uin, line)
      enddo
c.......................................................................
c
      ared = ared * rconversion
      bred = bred * rconversion
      cred = cred * rconversion
      salph = sin(alph * deg2rad)
      sbeta = sin(beta * deg2rad)
      sgamm = sin(gamm * deg2rad)
      calph = cos(alph * deg2rad)
      cbeta = cos(beta * deg2rad)
      cgamm = cos(gamm * deg2rad)
c
      if (.not.spgdone) call spgr (spg, uout, uout, ierr)
      call molbuildcell (uout)
c
c.....write general information on the crystal:
c
      write (uout,805) title(1:leng(title))
      write (uout,810) spg(1:leng(spg))
      write (uout,812) ared, bred, cred, alph, beta, gamm
      write (uout,813) calph, cbeta, cgamm
      vol = ared * bred * cred * sqrt (1d0 - calph*calph - cbeta*cbeta -
     &      cgamm*cgamm + 2d0 * calph * cbeta * cgamm)
      zmult = mcdvec (multneq, 1, nneq)
      write (uout,814) vol
      write (uout,817) zmult
      if (zmult .gt. 0) then
         write (uout,816) (namneq(i)(1:leng(namneq(i))),
     &                    dble(multneq(i))/dble(zmult), i = 1, nneq)
         xmass = 0d0
         xcharge = 0d0
         do i = 1, nneq
            xmass = xmass + atmweight(zzneq(i)) * multneq(i)
            xcharge = xcharge + qneq(i) * multneq(i)
         enddo
         xmass = xmass / zmult
         write (uout,818) vol/zmult, xmass, 11.206058*xmass*zmult/vol,
     &                    xcharge
      endif
      call preewald ()
      write (uout,820) nneq
      do i = 1, nneq
         vmad(i) = fewald(xneq(i),yneq(i),zneq(i),qneq(i),nshew,ierr)
         write (uout,822) i, xneq(i), yneq(i), zneq(i), qneq(i),
     &      multneq(i), namneq(i)(1:leng(namneq(i))), zzneq(i),
     &      vmad(i)
         if (ierr.ne.0) call error ('molcrystal', 'Error in Madelung',
     &                              warning)
      enddo
c
c.....Determine Madelung constant at each ionic position and average
c     Madelung constant for the crystal:
c
      if (nneq .gt. 1) then
         vscale = (vol/zmult) ** (1d0/3d0)
         zmin = 1d10
         do i = 1, nneq
            if (abs(qneq(i)) .gt. 1d-1) zmin = min(zmin, abs(qneq(i)))
         enddo
         avgcmadelung = 0d0
         do i = 1, nneq
            vmad(i) = -vmad(i) * vscale / zmin
            avgcmadelung = avgcmadelung +
     &                     (qneq(i)/zmin) * vmad(i) * multneq(i)
         enddo
         avgcmadelung = -0.5d0 * avgcmadelung / zmult
*mdc*if debug
         write (uout,830)
         do i = 1, nneq
            write (uout,832) i, namneq(i)(1:leng(namneq(i))),
     &                       vmad(i)
         enddo
*mdc*endif
         write (uout,834) avgcmadelung
         write (uout,836) zmin, vscale
         write (uout,838) -avgcmadelung * zmin * zmin / vscale
      endif
c
c.....write complete specification of crystal, cluster and hole:
c
      write (uout,840) ncell
      write (uout,842) (i, xcell(i), ycell(i), zcell(i), typcell(i),
     &                 namneq(typcell(i))(1:leng(namneq(typcell(i)))),
     &                 zzneq(typcell(i)),
     &                 i = 1, ncell)
      write (uout,850) nclus
      write (uout,852) (i, xclus(i), yclus(i), zclus(i), typclus(i),
     &                 namclus(typclus(i))(1:leng(namclus(typclus(i)))),
     &                 zzclus(typclus(i)),
     &                 i = 1, nclus)
      write (uout,860) nhole
      write (uout,862) (i, xhole(i), yhole(i), zhole(i), i = 1, nhole)
c
c.....FORMATS:
c
 800  format (/
     & 1x, 'MOLCRYSTAL: Reading molecular system description.')
 805  format (
     & 1x, 'Run title: ', a)
 810  format (
     & 1x, 'Space group symbol: ', a)
 812  format (
     & 1x, 'Cell parameters (a..c): ', 3f12.6, ' bohr'/
     & 1x, 'Cell angles (degrees):  ', 3f12.6)
 813  format (
     & 1x, 'Director cosines:       ', 3f12.6)
 814  format (
     & 1x, 'Cell volume:            ', f12.6, ' bohr^3')
 816  format (
     & 1x, 'Molecular formula:      ', (1x,5(a6,'(',f6.3,')',1x)))
 817  format (
     & 1x, 'Formulas per cell:      ', i6)
 818  format (
     & 1x, 'Molecular volume:       ', f12.6, ' bohr^3'/
     & 1x, 'Crystal formula mass:   ', f12.6, ' g/mol'/
     & 1x, 'Crystal density:        ', f12.6, ' g/cm^3'/
     & 1x, 'Unit cell charge:       ', f12.6)
 820  format (/
     & 1x, 'Nonequivalent atoms in the unit cell: ', i5/
     & 1x, '-i-----x----------y----------z----------q-----mul-name-'
     & , '-At.N------Madelung----')
 822  format (i3, 4f11.6, i3, a6, i5, 1p, e17.8)
*mdc*if debug
 830  format (/1x, '*** MADELUNG CONSTANTS AT THE IONIC SITES :'/
     & 1x, '--ION----NAME-------C.MADELUNG')
 832  format (i6, a8, 1p, e17.8)
*mdc*endif
 834  format (/1x, 'Average Madelung constant for the crystal: ',
     &        1p, e17.8)
 836  format (1x, 'Scaling terms (zmin, vscale): ', 1p, 2e17.8)
 838  format (1x, 'Electrostatic energy: ', 1p, e17.8, ' atomic units')
 840  format (/
     & 1x, 'Total number of atoms in the unit cell: ', i5/
     & 1x, '-i------x-----------y-----------z-----type--name--At.N-')
 842  format (i3, 3f12.6, i3, a6, i6)
 850  format (/
     & 1x, 'Total number of cluster atoms: ', i5/
     & 1x, '-i------x-----------y-----------z-----type--name--At.N-')
 852  format (i3, 3f12.6, i3, a6, i6)
 860  format (/
     & 1x, 'Total number of hole positions: ', i5/
     & 1x, '-i------x-----------y-----------z-----')
 862  format (i3, 3f12.6)
c
      end
