c
c-----------------------------------------------------------------------
c
      subroutine molnormal ()
c
c.....molnormal - determines the normal vector components for all faces
c     of polyhedra. The normal is defined positive towards the center
c     of the coresponding polyhedron. The normal vector is normalized
c     to 1.
c
c     Let A=gnormal(1,); B=gnormal(2,); C=gnormal(3,); D=gnormal(4,)
c     The equation for the face plane is:
c
c           A x + B y + C z + D = 0
c
c     The normal vector to the plane is (A,B,C).
c
c GLOBAL DATA:
c
      include           'implicit.inc'
      include           'molge.inc'
c
c LOCAL VARIABLES:
c
      real              xm, ym, zm, x1, y1, z1, x2, y2, z2, x3, y3, z3,
     &                  hh, hh1, hh2
      integer           i, j, j1
c
      do i = 1, gnplh
         xm = gxpt(gctplh(i))
         ym = gypt(gctplh(i))
         zm = gzpt(gctplh(i))
         do j = 1, gnface(i)
            j1 = giface(j,i)
            x1 = gxpt(giside(1,j1))
            y1 = gypt(giside(1,j1))
            z1 = gzpt(giside(1,j1))
            x2 = gxpt(giside(2,j1))
            y2 = gypt(giside(2,j1))
            z2 = gzpt(giside(2,j1))
            x3 = gxpt(giside(3,j1))
            y3 = gypt(giside(3,j1))
            z3 = gzpt(giside(3,j1))
            gnormal(1,j1) = (y3-y2)*z1 + (y1-y3)*z2 + (y2-y1)*z3
            gnormal(2,j1) = (z3-z2)*x1 + (z1-z3)*x2 + (z2-z1)*x3
            gnormal(3,j1) = (x3-x2)*y1 + (x1-x3)*y2 + (x2-x1)*y3
            gnormal(4,j1) = x1*(y3*z2-y2*z3) + y1*(x2*z3-x3*z2) +
     &                      z1*(x3*y2-x2*y3)
            hh = gnormal(1,j1)*xm + gnormal(2,j1)*ym +
     &           gnormal(3,j1)*zm + gnormal(4,j1)
*mdc*if normalize
            hh1 = sqrt(gnormal(1,j1)*gnormal(1,j1) +
     &                 gnormal(2,j1)*gnormal(2,j1) +
     &                 gnormal(3,j1)*gnormal(3,j1))
            hh2 = sign (1e0/hh1, hh)
            gnormal(1,j1) = hh2 * gnormal(1,j1)
            gnormal(2,j1) = hh2 * gnormal(2,j1)
            gnormal(3,j1) = hh2 * gnormal(3,j1)
            gnormal(4,j1) = hh2 * gnormal(4,j1)
*mdc*else
*            if (hh .lt. 0.0) then
*               gnormal(1,j1) = - gnormal(1,j1)
*               gnormal(2,j1) = - gnormal(2,j1)
*               gnormal(3,j1) = - gnormal(3,j1)
*               gnormal(4,j1) = - gnormal(4,j1)
*            endif
*mdc*endif
         enddo
      enddo
      end
