c
c-----------------------------------------------------------------------
c
      subroutine molplh (
     &  atom,          !Atom at the center of polyhedra
     &  posselect,     !Choose polyhedra center by position not by name
     &  xct, yct, zct, !Polyhedron center
     &  useautodist,   !Select authomatically the limit bond distances
     &  dmin, dmax,    !Limit distances otherwise
     &  nneig,         !Select type of atoms at the vertices
     &  namneig,       !Names of possible vertex neighbors
     &  usedefrgb,     !Use default colors?
     &  rgb,           !Use this color instead
     &  itexture,      !POV texture index
     &  dofaces,       !Plot the ployhedra as faces
     &  doedges,       !Plot the face edges
     &  stickrad,      !Edges radii
     &  uout)
c
c.....molplh - Create coordination polyhedra around an atom or an
c     crystallographic position.
c
      include           'implicit.inc'
      include           'strtools.inc'
      include           'consts.inc'
      include           'datatm.inc'
      include           'error.inc'
      include           'molec.inc'
      include           'molge.inc'
c
      character*(*)     atom, namneig(*)
      integer           nneig, uout, itexture
      real*8            xct, yct, zct, dmin, dmax, rgb(4), stickrad
      logical           posselect, useautodist, usedefrgb
      logical           dofaces, doedges
c
      integer           i, atmZ
      real*8            qi, defdmin, defdmax, defrgb(4)
      logical           doit, done
      character*(3)     name
c
      character*(6)     uppcase
      logical           equal
      integer           zhlookup
c
c.....The atomic number and atom properties must be known if the
c     authomatic procedures are invoked:
c
      if (useautodist .or. usedefrgb) then
         name = atom
         name = uppcase(name)
         if (name(2:2).lt.'A' .or. name(2:2).gt.'Z') then
            name(2:2) = ' '
         endif
         name(3:3) = null
         atmZ = zhlookup (name)
         if (atmZ .le. 0) call error ('molplh',
     &      'Atom name not recognized in POLYHEDRON order!', faterr)
*mdc*if debug
*         write (uout,900) name(1:2), atmZ
*mdc*endif
      endif
c
      if (usedefrgb) then
         defrgb(1) = dble(CPKcol(1,atmCPK(atmZ))) / 255d0
         defrgb(2) = dble(CPKcol(2,atmCPK(atmZ))) / 255d0
         defrgb(3) = dble(CPKcol(3,atmCPK(atmZ))) / 255d0
         defrgb(4) = 0d0
      else
         defrgb(1) = rgb(1)
         defrgb(2) = rgb(2)
         defrgb(3) = rgb(3)
         defrgb(4) = rgb(4)
      endif
c
c.....The atom at the center of the polyhedron must be known, even
c     when the center has been chosen by position. If no atom exists
c     at the entered position, a dummy atom with null charge and radius
c     will be assumed:
c
      if (useautodist .and. posselect) then
         i = 1
         done = .false.
         do while (i.le.nmol .and. .not.done)
            if (abs(xct-xmol(i)).le.EPS .and.
     &          abs(yct-ymol(i)).le.EPS .and.
     &          abs(zct-zmol(i)).le.EPS      ) then
               done = .true.
            else
               i = i + 1
            endif
         enddo
         if (done) then
            if (codemol(i).eq.codecrystal) then
               qi = qneq(typmol(i))
            else
               qi = qclus(typmol(i))
            endif
         else
            qi = 0d0
         endif
         defdmin = 0d0
         if (typrad .eq. typION) then
            defdmax = (atmION(qi,atmZ)/bohr2pm) * ionfactor(qi)
         else if (typrad .eq. typvdW) then
            defdmax = (atmvdW(atmZ)/bohr2pm) * ionfactor(qi)
         else if (typrad .eq. typcov) then
            defdmax = (atmcov(atmZ)/bohr2pm) * ionfactor(qi)
         endif
      else
         defdmin = dmin
         defdmax = dmax
      endif
c
c.....Call now to molplbuild to get the coordination polyhedra:
c
      if (posselect) then
c
c........The polyhedron center has been fixed by position:
c
         call molplbuild (xct, yct, zct, useautodist, defdmin, defdmax,
     &                    nneig, namneig,
     &                    usedefrgb, defrgb, itexture,
     &                    dofaces, doedges, stickrad, uout)
      else
c
c........One or more polyhedra will be created, the center being
c        chosen by its name:
c........Search for the adequate atom types:
c
         do i = 1, nmol
            if (codemol(i).eq.codecrystal) then
               doit = equal(namneq(typmol(i)),atom)
               qi = qneq(typmol(i))
            else
               doit = equal(namclus(typmol(i)),atom)
               qi = qclus(typmol(i))
            endif
            doit = doit .and. inclip(i)
            if (doit) then
               if (useautodist) then
                  defdmin = 0d0
                  if (typrad .eq. typION) then
                     defdmax = (atmION(qi,atmZ)/bohr2pm)*ionfactor(qi)
                  else if (typrad .eq. typvdW) then
                     defdmax = (atmvdW(atmZ)/bohr2pm) * ionfactor(qi)
                  else if (typrad .eq. typcov) then
                     defdmax = (atmcov(atmZ)/bohr2pm) * ionfactor(qi)
                  endif
               endif
               xct = xmol(i)
               yct = ymol(i)
               zct = zmol(i)
               call molplbuild (xct, yct, zct,
     &                          useautodist, defdmin, defdmax,
     &                          nneig, namneig,
     &                          usedefrgb, defrgb, itexture,
     &                          dofaces, doedges, stickrad, uout)
            endif
         enddo
      endif
c
*mdc*if debug
* 900  format (1x, 'DBG(molplh.1). Atom:', a, i5)
*mdc*endif
      end
