c
c-----------------------------------------------------------------------
c
      subroutine molstick (
     &  atom1, atom2,  !Name of the atoms to be connected
     &  useautodist,   !Select authomatically the limit bond distances
     &  dmin, dmax,    !Limit distances otherwise
     &  stickrad,      !Stick radius
     &  dobicolor,     !Create bicolor bonds?
     &  usedefrgb,     !Use default colors?
     &  rgb,           !Use this color instead
     &  rgb2,          !Use this color for the second atom if bicolor
     &  itexture,      !POV texture index
     &  uout)
c
c.....molstick - link atoms
c
      include           'implicit.inc'
      include           'consts.inc'
      include           'datatm.inc'
      include           'error.inc'
      include           'molec.inc'
      include           'molge.inc'
c
      character*(*)     atom1, atom2
      integer           uout, itexture
      real*8            dmin, dmax, stickrad, rgb(4), rgb2(4)
      logical           useautodist, usedefrgb, dobicolor
c
      integer           i, i0, i1, j
      real*8            defdmin, defdmax, defradius
      real*8            defrgb(4), defrgb2(4)
      real*8            xx, yy, zz, d2, x1, y1, z1, x2, y2, z2
      real*8            ri, rj
      integer           qi, qj, atmZi, atmZj
      logical           equal, is1, is2, doit
c
      i0 = gnpt
      i1 = gnstk
c
      defdmin = dmin*dmin
      defdmax = dmax*dmax
      defradius = stickrad
c
c.....Setting default colors:
c
      if (usedefrgb) then         !Use brown color for the bonds
         defrgb(1) = 0.647059d0
         defrgb(2) = 0.164706d0
         defrgb(3) = 0.164706d0
         defrgb(4) = 0.0d0
      else
         defrgb(1) = rgb(1)
         defrgb(2) = rgb(2)
         defrgb(3) = rgb(3)
         defrgb(4) = rgb(4)
         defrgb2(1) = rgb2(1)
         defrgb2(2) = rgb2(2)
         defrgb2(3) = rgb2(3)
         defrgb2(4) = rgb2(4)
      endif
c
c.....Search for the adequate atom types:
c
      do i = 1, nmol-1
         if (codemol(i).eq.codecrystal) then
            is1 = equal(namneq(typmol(i)),atom1)
            is2 = equal(namneq(typmol(i)),atom2)
            qi = nint(qneq(typmol(i)))
            atmZi = zzneq(typmol(i))
         else
            is1 = equal(namclus(typmol(i)),atom1)
            is2 = equal(namclus(typmol(i)),atom2)
            qi = nint(qclus(typmol(i)))
            atmZi = zzclus(typmol(i))
         endif
         if (useautodist) then
            if (typrad .eq. typION) then
               ri = (atmION(qi,atmZi)/bohr2pm) * ionfactor(qi)
            else if (typrad .eq. typvdW) then
               ri = (atmvdW(atmZi)/bohr2pm) * ionfactor(qi)
            else if (typrad .eq. typcov) then
               ri = (atmcov(atmZi)/bohr2pm) * ionfactor(qi)
            endif
         endif
         do j = i+1, nmol
            if (codemol(j).eq.codecrystal) then
               doit = (is1 .and. equal(namneq(typmol(j)),atom2)) .or.
     &                (is2 .and. equal(namneq(typmol(j)),atom1))
               qj = nint(qneq(typmol(j)))
               atmZj = zzneq(typmol(j))
            else
               doit = (is1 .and. equal(namclus(typmol(j)),atom2)) .or.
     &                (is2 .and. equal(namclus(typmol(j)),atom1))
               qj = nint(qclus(typmol(j)))
               atmZj = zzclus(typmol(j))
            endif
c
c...........One of the atoms, at least, must be inside the graphical
c           clipping box:
c
            doit = doit .and. (inclip(i) .or. inclip(j))
c
c...........Analyze if the atom distances are in the right range:
c
            if (doit) then
               if (useautodist) then
                  if (typrad .eq. typION) then
                     rj = (atmION(qj,atmZj)/bohr2pm) * ionfactor(qj)
                  else if (typrad .eq. typvdW) then
                     rj = (atmvdW(atmZj)/bohr2pm) * ionfactor(qj)
                  else if (typrad .eq. typcov) then
                     rj = (atmcov(atmZj)/bohr2pm) * ionfactor(qj)
                  endif
                  defdmin = 0d0
                  defdmax = (ri + rj)**2
*mdc*if debug
*                  write (uout,905) qi, qj, ri, rj, defdmax
*mdc*endif
               endif
               xx = (xmol(i)-xmol(j)) * ared
               yy = (ymol(i)-ymol(j)) * bred
               zz = (zmol(i)-zmol(j)) * cred
               d2 = xx*xx + yy*yy + zz*zz + 2d0 *
     &              (xx*yy*cgamm + yy*zz*calph + zz*xx*cbeta)
               doit = (d2.ge.defdmin) .and. (d2.le.defdmax)
            endif
c
c...........If all requeriments are fulfilled add the new stick:
c
            if (doit .and. .not.dobicolor) then
               if (gnpt+2.gt.GMPT) call error ('molstick',
     &            'Too many points. Increase GMPT', faterr)
               if (gnstk+1.gt.GMSTK) call error ('molstick',
     &            'Too many sticks. Increase GMSTK', faterr)
               gnstk = gnstk + 1
               gstkpt(1,gnstk) = gnpt + 1
               gstkpt(2,gnstk) = gnpt + 2
               gstkrad(gnstk) = defradius
               gstkdefrgb(gnstk) = usedefrgb
               gstkrgbf(1,gnstk) = defrgb(1)
               gstkrgbf(2,gnstk) = defrgb(2)
               gstkrgbf(3,gnstk) = defrgb(3)
               gstkrgbf(4,gnstk) = defrgb(4)
               gstktext(gnstk) = itexture
               x1 = xmol(i)
               y1 = ymol(i)
               z1 = zmol(i)
               x2 = xmol(j)
               y2 = ymol(j)
               z2 = zmol(j)
               if (.not. inclip(i)) then
                  call molcliplin (x2, y2, z2, x1, y1, z1,
     &                 x0grfclip, x1grfclip, y0grfclip, y1grfclip,
     &                 z0grfclip, z1grfclip)
               else if (.not. inclip(j)) then
                  call molcliplin (x1, y1, z1, x2, y2, z2,
     &                 x0grfclip, x1grfclip, y0grfclip, y1grfclip,
     &                 z0grfclip, z1grfclip)
               endif
               gnpt = gnpt + 1
               gxpt(gnpt) = x1
               gypt(gnpt) = y1
               gzpt(gnpt) = z1
               gnpt = gnpt + 1
               gxpt(gnpt) = x2
               gypt(gnpt) = y2
               gzpt(gnpt) = z2
            else if (doit) then
               if (gnpt+3.gt.GMPT) call error ('molstick',
     &            'Too many points. Increase GMPT', faterr)
               if (gnstk+2.gt.GMSTK) call error ('molstick',
     &            'Too many sticks. Increase GMSTK', faterr)
               gnstk = gnstk + 1
               gstkpt(1,gnstk) = gnpt + 1
               gstkpt(2,gnstk) = gnpt + 2
               gstkrad(gnstk) = defradius
               gstkdefrgb(gnstk) = usedefrgb
               gstkrgbf(1,gnstk) = defrgb(1)
               gstkrgbf(2,gnstk) = defrgb(2)
               gstkrgbf(3,gnstk) = defrgb(3)
               gstkrgbf(4,gnstk) = defrgb(4)
               gstktext(gnstk) = itexture
               gnstk = gnstk + 1
               gstkpt(1,gnstk) = gnpt + 2
               gstkpt(2,gnstk) = gnpt + 3
               gstkrad(gnstk) = defradius
               gstkdefrgb(gnstk) = usedefrgb
               gstkrgbf(1,gnstk) = defrgb2(1)
               gstkrgbf(2,gnstk) = defrgb2(2)
               gstkrgbf(3,gnstk) = defrgb2(3)
               gstkrgbf(4,gnstk) = defrgb2(4)
               gstktext(gnstk) = itexture
               if (is1) then
                  x1 = xmol(i)
                  y1 = ymol(i)
                  z1 = zmol(i)
                  x2 = xmol(j)
                  y2 = ymol(j)
                  z2 = zmol(j)
               else
                  x1 = xmol(j)
                  y1 = ymol(j)
                  z1 = zmol(j)
                  x2 = xmol(i)
                  y2 = ymol(i)
                  z2 = zmol(i)
               endif
               xx = 0.5d0 * (x1 + x2)
               yy = 0.5d0 * (y1 + y2)
               zz = 0.5d0 * (z1 + z2)
               gnpt = gnpt + 1
               gxpt(gnpt) = x1
               gypt(gnpt) = y1
               gzpt(gnpt) = z1
               gnpt = gnpt + 1
               gxpt(gnpt) = xx
               gypt(gnpt) = yy
               gzpt(gnpt) = zz
               gnpt = gnpt + 1
               gxpt(gnpt) = x2
               gypt(gnpt) = y2
               gzpt(gnpt) = z2
            endif
         enddo
      enddo
c
c.....transform points to cartesian coordinates:
c
      do i = i0 + 1, gnpt
         gxpt(i) = gxpt(i)*crys2car(1) + gypt(i)*crys2car(2) +
     &             gzpt(i)*crys2car(3)
         gypt(i) = gypt(i)*crys2car(4) + gzpt(i)*crys2car(5)
         gzpt(i) = gzpt(i)*crys2car(6)
      enddo
c
      if (uout.gt.0) write (uout,810) gnstk-i1
c
c.....FORMATS:
c
 810  format (
     & 1x, 'MOLSTICK: ', i6,
     & ' sticks added to the molecular graphics database.')
*mdc*if debug
* 900  format (1x, 'DBG(molstick.1). Atom:', a, i5, ' Atom:', a, i5)
* 905  format (1x, 'DBG(molstick.2). Charges:', 2i4,
*     &        ' Distances:', 3f12.6)
*mdc*endif
c
      end
