c
c-----------------------------------------------------------------------
c
      subroutine triangulate (level, phaseth, phaseph, uout)
c
c.....triangulate - produce OFF files describing the attraction
c     basins of all the non-equivalent ion types.
c
      include           'implicit.inc'
c
      integer           level, uout
      real*8            phaseth, phaseph
c
c.....Polyhedra data structure:
c
      include           'polyhedron.inc'
c
      integer           ix(mvert), iy(mvert), iz(mvert) !vert. coord.
      integer           i1, i2, i3, i4, i5, i6, ilev, i, j, nf1
      integer           ix4, iy4, iz4, ix5, iy5, iz5, ix6, iy6, iz6
      integer           nvertold, startpoly
      logical           new, project
      real*8            xpt, ypt, zpt
      real*8            rr
      real*8            theta, phi, stheta
c
      if (level .lt. 0) then
         project = .true.
         level = -level
      else
         project = .false.
      endif
      if (level .lt. 100) then
         startpoly = 1
      else
         level = level - 100
         startpoly = 2
      endif
c
c.....First, let us compute a convex polyhedra that approximately
c     triangulates the sphere. We will start with a octahedron and
c     succesively divide each triangular facet into four new ones.
c     The coordinates for the vertices will be maintained as integer
c     values to get best precision and efficiency.
c
      nvf = 3         !Each face will be a triangle
c
      if (startpoly.eq.1) then    !Start with an octahedron
         nvert = 6
         ix(1) =  1024
         iy(1) =     0
         iz(1) =     0
         ix(2) =     0
         iy(2) =  1024
         iz(2) =     0
         ix(3) =     0
         iy(3) =     0
         iz(3) =  1024
         ix(4) = -1024
         iy(4) =     0
         iz(4) =     0
         ix(5) =     0
         iy(5) = -1024
         iz(5) =     0
         ix(6) =     0
         iy(6) =     0
         iz(6) = -1024
c
c.....store initial faces:
c
         isw0 = 1
         isw1 = 0
         nface(0) = 8
         iface(1,1,0) = 1
         iface(2,1,0) = 2
         iface(3,1,0) = 3
         iface(1,2,0) = 1
         iface(2,2,0) = 3
         iface(3,2,0) = 5
         iface(1,3,0) = 5
         iface(2,3,0) = 3
         iface(3,3,0) = 4
         iface(1,4,0) = 2
         iface(2,4,0) = 4
         iface(3,4,0) = 3
c
         iface(1,5,0) = 6
         iface(2,5,0) = 1
         iface(3,5,0) = 5
         iface(1,6,0) = 6
         iface(2,6,0) = 2
         iface(3,6,0) = 1
         iface(1,7,0) = 6
         iface(2,7,0) = 4
         iface(3,7,0) = 2
         iface(1,8,0) = 6
         iface(2,8,0) = 5
         iface(3,8,0) = 4
c
      else               ! Start with a tetrahedron
         nvert = 4
         ix(1) =  1024
         iy(1) =  1024
         iz(1) =  1024
         ix(2) = -1024
         iy(2) = -1024
         iz(2) =  1024
         ix(3) = -1024
         iy(3) =  1024
         iz(3) = -1024
         ix(4) =  1024
         iy(4) = -1024
         iz(4) = -1024
c
         isw0 = 1
         isw1 = 0
         nface(0) = 4
         iface(1,1,0) = 1
         iface(2,1,0) = 4
         iface(3,1,0) = 3
         iface(1,2,0) = 2
         iface(2,2,0) = 4
         iface(3,2,0) = 1
         iface(1,3,0) = 1
         iface(2,3,0) = 3
         iface(3,3,0) = 2
         iface(1,4,0) = 2
         iface(2,4,0) = 3
         iface(3,4,0) = 4
c
      endif
c
c.....In PROJECT mode get cartesian coordinates projected out to the
c     surface of the sphere:
c
      if (project) then
         do j = 1, nvert
            xpt = dble(ix(j))
            ypt = dble(iy(j))
            zpt = dble(iz(j))
            rr = sqrt(xpt*xpt + ypt*ypt + zpt*zpt)
            theta = acos(zpt/rr) + phaseth
            phi = atan2(ypt,xpt) + phaseph
            stheta = sin(theta)
            sthcph(j) = stheta * cos(phi)
            sthsph(j) = stheta * sin(phi)
            cth(j) = cos(theta)
            xvert(j) = radius * sthcph(j)
            yvert(j) = radius * sthsph(j)
            zvert(j) = radius * cth(j)
         enddo
      endif
c
      do ilev = 1, min(level,mlevel)
c
c........exchange bins for old and new levels:
c
         nvertold = nvert
         isw0 = mod(isw0+1,2)
         isw1 = mod(isw0+1,2)
         nface(isw1) = 0
c
c........iterate over the facets of the old level:
c
         do i = 1, nface(isw0)
            i1 = iface(1,i,isw0)
            i2 = iface(2,i,isw0)
            i3 = iface(3,i,isw0)
c
c...........get new points to divide this triangle:
c
            ix4 = (ix(i1)+ix(i2))/2
            iy4 = (iy(i1)+iy(i2))/2
            iz4 = (iz(i1)+iz(i2))/2
            ix5 = (ix(i2)+ix(i3))/2
            iy5 = (iy(i2)+iy(i3))/2
            iz5 = (iz(i2)+iz(i3))/2
            ix6 = (ix(i3)+ix(i1))/2
            iy6 = (iy(i3)+iy(i1))/2
            iz6 = (iz(i3)+iz(i1))/2
c
c...........store only new points:
c
            new = .true.
            i4 = 0
            do while (new .and. i4.lt.nvert)
               i4 = i4 + 1
               if (ix(i4) .eq. ix4) then
                  if (iy(i4) .eq. iy4) then
                     if (iz(i4) .eq. iz4) then
                        new = .false.
                     endif
                  endif
               endif
            enddo
            if (new) then
               nvert = nvert + 1
               i4 = nvert
               ix(i4) = ix4
               iy(i4) = iy4
               iz(i4) = iz4
               if (project) then
                  xvert(i4) = 0.5d0 * (xvert(i1)+xvert(i2))
                  yvert(i4) = 0.5d0 * (yvert(i1)+yvert(i2))
                  zvert(i4) = 0.5d0 * (zvert(i1)+zvert(i2))
               endif
            endif
            new = .true.
            i5 = 0
            do while (new .and. i5.lt.nvert)
               i5 = i5 + 1
               if (ix(i5) .eq. ix5) then
                  if (iy(i5) .eq. iy5) then
                     if (iz(i5) .eq. iz5) then
                        new = .false.
                     endif
                  endif
               endif
            enddo
            if (new) then
               nvert = nvert + 1
               i5 = nvert
               ix(i5) = ix5
               iy(i5) = iy5
               iz(i5) = iz5
               if (project) then
                  xvert(i5) = 0.5d0 * (xvert(i2)+xvert(i3))
                  yvert(i5) = 0.5d0 * (yvert(i2)+yvert(i3))
                  zvert(i5) = 0.5d0 * (zvert(i2)+zvert(i3))
               endif
            endif
            new = .true.
            i6 = 0
            do while (new .and. i6.lt.nvert)
               i6 = i6 + 1
               if (ix(i6) .eq. ix6) then
                  if (iy(i6) .eq. iy6) then
                     if (iz(i6) .eq. iz6) then
                        new = .false.
                     endif
                  endif
               endif
            enddo
            if (new) then
               nvert = nvert + 1
               i6 = nvert
               ix(i6) = ix6
               iy(i6) = iy6
               iz(i6) = iz6
               if (project) then
                  xvert(i6) = 0.5d0 * (xvert(i3)+xvert(i1))
                  yvert(i6) = 0.5d0 * (yvert(i3)+yvert(i1))
                  zvert(i6) = 0.5d0 * (zvert(i3)+zvert(i1))
               endif
            endif
c
c...........store the four new faces that result from the division of
c           the old face:
c
            nf1 = nface(isw1)
            iface(1,nf1+1,isw1) = i1
            iface(2,nf1+1,isw1) = i6
            iface(3,nf1+1,isw1) = i4
            iface(1,nf1+2,isw1) = i6
            iface(2,nf1+2,isw1) = i5
            iface(3,nf1+2,isw1) = i4
            iface(1,nf1+3,isw1) = i6
            iface(2,nf1+3,isw1) = i3
            iface(3,nf1+3,isw1) = i5
            iface(1,nf1+4,isw1) = i4
            iface(2,nf1+4,isw1) = i5
            iface(3,nf1+4,isw1) = i2
            nface(isw1) = nf1 + 4
         enddo
c
c........In PROJECT mode project out the new points on the sphere
c        surface:
c
         if (project) then
            do j = nvertold+1, nvert
               xpt = xvert(j)
               ypt = yvert(j)
               zpt = zvert(j)
               rr = sqrt(xpt*xpt + ypt*ypt + zpt*zpt)
               theta = acos(zpt/rr) + phaseth
               phi = atan2(ypt,xpt) + phaseph
               stheta = sin(theta)
               sthcph(j) = stheta * cos(phi)
               sthsph(j) = stheta * sin(phi)
               cth(j) = cos(theta)
               xvert(j) = radius * sthcph(j)
               yvert(j) = radius * sthsph(j)
               zvert(j) = radius * cth(j)
            enddo
         endif
c
      enddo
c
c.....The final set of faces is in the isw1 bin.
c.....Determine the spherical coordinates of the vertices, and store
c     sin(theta)sin(phi), sin(theta)cos(phi), and cos(theta):
c
      if (project) return
      do j = 1, nvert
         xpt = dble(ix(j))
         ypt = dble(iy(j))
         zpt = dble(iz(j))
         rr = sqrt(xpt*xpt + ypt*ypt + zpt*zpt)
         theta = acos(zpt/rr) + phaseth
         phi = atan2(ypt,xpt) + phaseph
         stheta = sin(theta)
         sthcph(j) = stheta * cos(phi)
         sthsph(j) = stheta * sin(phi)
         cth(j) = cos(theta)
      enddo
c
c.....Determine cartesian coordinates of the points in the surface of the
c     sphere:
c
      do j = 1, nvert
         xvert(j) = radius * sthcph(j)
         yvert(j) = radius * sthsph(j)
         zvert(j) = radius * cth(j)
      enddo
c
      if (startpoly .eq. 1) then
         write (uout,100) level, phaseth, phaseph
      else
         write (uout,101) level, phaseth, phaseph
      endif
      write (uout,105) nvert, nface(isw1)
 100  format (/
     & 1x, 'TRIANGULATE: Tesselation of the sphere by iterative'/
     & 1x, '             division of the faces of an octahedron.'//
     & 1x, 'Iteration level: ', i4/
     & 1x, 'Phase shift for theta coordinate: ', 1p, e12.4, 0p/
     & 1x, 'Phase shift for phi   coordinate: ', 1p, e12.4, 0p )
 101  format (/
     & 1x, 'TRIANGULATE: Tesselation of the sphere by iterative'/
     & 1x, '             division of the faces of an tetrahedron.'//
     & 1x, 'Iteration level: ', i4/
     & 1x, 'Phase shift for theta coordinate: ', 1p, e12.4, 0p/
     & 1x, 'Phase shift for phi   coordinate: ', 1p, e12.4, 0p )
 105   format (
     & 1x, 'Number of vertices created: ', i6/
     & 1x, 'Number of triangular faces: ', i6)
c
      end
