c
c-----------------------------------------------------------------------
c
      subroutine wr2ddbasin (hoffset, voffset, datafile, gnufile, uout)
c
c.....wr2ddbasin - read the polyhedron description as an DBASIN file
c     and prepare a gnuplot file to plot the scalar property along each
c     ray as a different 2D line.
c
      include           'implicit.inc'
      include           'error.inc'
      include           'polyhedron.inc'
      include           'strtools.inc'
c
      real*8            hoffset, voffset
      character*(*)     datafile, gnufile
      integer           uout
c
      integer           lur, luw, j, k
c
      integer           mpoint
      parameter         (mpoint = 500)
      real*8            lrho(0:500), hoff, voff
      real*8            x, y, z, xnuc, ynuc, znuc, x1, y1, z1
      real*8            rdist, rdelta
      integer           nedge, npoint
      character*(80)    line
c
      logical           commentline
      integer           leng, fopen
c
c.....Open the input and output files:
c
      if (fopen (lur, datafile, ioread) .eq. ioerror) then
         write (uout,*) 'wr2ddbasin: Data file is ',
     &               datafile(1:leng(datafile))
         call error ('wr2ddbasin', 'I/O error opening DBASIN file',
     &               faterr)
         return
      endif
      if (fopen (luw, gnufile, iowrite) .eq. ioerror) then
         write (uout,*) 'wr2ddbasin: Output file is ',
     &               gnufile(1:leng(gnufile))
         call error ('wr2ddbasin', 'I/O error opening GNUPlot file',
     &               faterr)
         return
      endif
c
      nvf = 0
      isw1 = 0
c
c.....Skip initial comment lines, if any:
c
      do while (commentline(lur, line))
         write (luw,600) line(1:leng(line))
      enddo
c
c.....Start reading:
c
      read (lur,*) nvert, nface(isw1), nedge
      read (lur,*) npoint, xnuc, ynuc, znuc, lrho(0)
      if (npoint .gt. mpoint) call error ('wr2ddbasin',
     &   'Too many density values per point. Increase MPOINT!', faterr)
c
      hoff = 0d0
      voff = 0d0
      do j = 1, nvert
c
c........Get the vertex coordinates and the density values:
c
         read (lur,*) x, y, z, (lrho(k), k = 1, npoint)
c
c........Pass the coordinates to spherical:
c
         x1 = x - xnuc
         y1 = y - ynuc
         z1 = z - znuc
         rdist = sqrt(x1*x1 + y1*y1 + z1*z1)
         rdelta = rdist / npoint
         do k = 0, npoint
            write (luw,605) hoff+k*rdelta, voff+lrho(k)
         enddo
         write (luw,610)
         hoff = hoff + hoffset
         voff = voff + voffset
      enddo
c
      call fclose (luw)
      call fclose (lur)
c
 600  format ('#', a)
 605  format (1p, 2e15.8)
 610  format (/)
c
      write (uout,100) datafile(1:leng(datafile)), nvert, npoint,
     &                 gnufile(1:leng(gnufile))
 100  format (/
     & 1x, 'WR2DDBASIN: Reading start points from a file'/
     & 1x, 'Input data file name: ', a/
     & 1x, 'Vertices read in:  ', i6/
     & 1x, 'Points per vertex: ', i6/
     & 1x, 'Gnuplot data file name: ', a)
c
      end
